@@ -10,12 +10,19 @@ package PDL::NiceSlice;
 #   $pdl->mslice(processed_args);
 #
 
-$PDL::NiceSlice::VERSION = 0.5;
+$PDL::NiceSlice::VERSION = 0.6;
 
 require PDL::Version; # get PDL version number
 if ("$PDL::Version::VERSION" !~ /cvs$/ and
     "$PDL::Version::VERSION" lt '2.2.2') { # make sure we got uptodate version
                                            # of nslice
+
+eval << 'EOF' unless PDL->can('flat');
+    sub PDL::flat { # fall through if < 2D
+      return $_[0]->dims == 1 ? $_[0] : $_[0]->clump(-1);
+    }
+EOF
+
 eval << 'EOH';
   {
     package PDL;
@@ -24,9 +31,6 @@ eval << 'EOH';
 		      die "multielement piddle where only one allowed" :
 			die "non piddle ref '".ref $_[0]."'"
 			  : int $_[0] }
-    sub PDL::flat { # fall through if < 2D
-      return $_[0]->getndims > 1 ? $_[0]->clump(-1) : $_[0];
-    }
     sub PDL::nslice {
       my($pdl) = shift;
       my @args = @_;
@@ -34,7 +38,7 @@ eval << 'EOH';
       for (@args) {
 	if (UNIVERSAL::isa($_,'PDL')) {
 	  if ($_->nelem > 1) {
-	    PDL::Core::barf 'piddle must be <= 1D' if $_->getndims > 1;
+	    PDL::Core::barf('piddle must be <= 1D') if $_->getndims > 1;
 	    # dice this axis
 	    $pdl = $pdl->dice_axis($i,$_);
 	    # and keep resulting dim fully in slice
@@ -56,6 +60,9 @@ eval << 'EOH';
     }
   }
 EOH
+
+# mark as lvalue sub if 5.6.x and above
+eval 'sub PDL::nslice : lvalue;' if ($^V and $^V ge v5.6.0);
 }
 
 use Text::Balanced; # used to find parenthesis-delimited blocks 
@@ -70,6 +77,15 @@ my @srcstr = (); # stack for refs to current source strings
 my $offset = 1;  # line offset
 my $file   = 'unknown';
 
+my $mypostfix = '';
+
+sub autosever {
+  my ($this,$arg) = @_;
+  $arg = 1 unless defined $arg;
+  if ($arg) {$mypostfix = '->sever'} else
+    {$mypostfix = ''}
+}
+
 sub line {
   die __PACKAGE__." internal error: can't determine line number"
     if $#srcstr < 0;
@@ -154,11 +170,11 @@ sub onearg ($) {
 # process the arg list
 sub procargs {
   my ($txt) = @_;
-  $txt =~ s/\((.*)\)/$1/;
+  $txt =~ s/\((.*)\)/$1/s;
   push @callstack, $txt; # for later error reporting
-  my $args = $txt =~ /^\s*$/ ? '' :
+  my $args = $txt =~ /^\s*$/s ? '' :
     join ',', map {onearg $_} splitprotected $txt, ',';
-  $args =~ s/\s//g; # get rid of whitespace
+  $args =~ s/\s//sg; # get rid of whitespace
   pop @callstack; # remove from call stack
   return "($args)";
 }
@@ -179,7 +195,7 @@ sub findslice {
     print STDERR "pass $ct: found slice expr $found at line ".line()."\n"
       if $verb;
     $processed .= "$prefix". ($prefix =~ /->$/ ? '' : '->').
-      'nslice'.procargs($found);
+      'nslice'.procargs($found).$mypostfix;
   }
   pop @srcstr; # clear stack
   $processed .= substr $src, pos($src); # append the remaining text portion
@@ -530,6 +546,11 @@ I<"not occurring between matched parentheses">.
 
 Source filtering can be switched on and off in scripts
 and perl modules by using or unloading C<PDL::NiceSlice>.
+
+Note: this will I<not> work in the L<perldl|perldl> shell
+(see below how to enable the new slicing syntax within L<perldl|perldl>).
+
+But now back to scripts and modules.
 Everything after C<use PDL::NiceSlice> will be translated
 and you can use the snew slicing syntax. Source filtering
 will continue until the end of the file is encountered.
@@ -574,6 +595,36 @@ Similarly, switch reporting off as needed
 Note that these commands will only work if you included
 the contents of F<local.perldlrc> in your perldl startup file.
 
+=head2 evals and C<PDL::NiceSlice>
+
+Due to C<PDL::NiceSlice> being a source filter it won't work
+in the usual way within evals. The following will I<not> do what
+you want:
+
+  $a = sequence 10;
+  eval << 'EOE';
+
+  use PDL::NiceSlice;
+  $b = $a(0:5);
+
+  EOE
+  print $b;
+
+Instead say:
+
+  use PDL::NiceSlice;
+  $a = sequence 10;
+  eval << 'EOE';
+
+  $b = $a(0:5);
+
+  EOE
+  print $b;
+
+Source filters I<must> be executed at compile time to be effective. And
+C<PDL::NiceFilter> is just a source filter (although it is not
+necessarily obvious for the casual user).
+
 =head1 Implementation
 
 C<PDL::NiceSlice> exploits the ability of Perl to use source filtering
